import express, { Response, Request, NextFunction, RequestHandler } from 'express';
import { Options as RateLimitOptions } from 'express-rate-limit';
declare type ExpressStaticParams = Parameters<typeof express.static>;
export interface StaticFiles {
    middlewares?: (string | Function)[];
    directory: string[];
    subdomain?: string;
    path: string;
    options?: ExpressStaticParams[1];
    rateLimitOptions?: RateLimitOptions;
    disableIndexRouter?: boolean;
    customStaticHandler?: (path: string) => RequestHandler[];
    indexFileName?: string;
}
export interface StaticFilesSubdomain extends StaticFiles {
    subdomain: string;
}
export interface ErroHandler {
    (req: Request, res: Response, next: NextFunction, error: any): Response<any>;
}
export interface NotFoundHandler {
    (req: Request, res: Response, next: NextFunction): Promise<Response<any>>;
}
export interface ConfigurationDebug {
    log?: Function;
    error?: Function;
}
export interface Configuration {
    rootFile: string;
    apiPrefix: string;
    debug: ConfigurationDebug;
    staticFiles?: StaticFilesSubdomain[] | StaticFiles;
    errorHandler?: ErroHandler;
    notFoundHandler?: NotFoundHandler;
}
export declare const defaultNotFoundHandler: NotFoundHandler;
export declare const defaultErrorHandler: ErroHandler;
export {};
/**
 * @module Configuration
 */
/**
 * @static
 * @member Configuration
 * @type {Configuration}
 *
 * @property {string} rootFile The file that is in the root of the app, for development
 * that should be in ./src/index.ts, this should always be '__filename'
 * @property {string} apiPrefix The prefix which will be used for the api e.g. /api
 * @property {module:Configuration.ConfigurationDebug} debug The debug functions
 * @property {?(Array<module:Configuration.StaticFilesSubdomain> | module:Configuration.StaticFiles)} staticFiles
 * Static files is used to serve static files
 * @property {?module:Configuration.ErroHandler} errorHandler All the thrown errors and the ones
 * from next() will be caught in the errorHandler.
 * @property {?module:Configuration.NotFoundHandler} notFoundHandler This handler is used when the
 * route is not found.
 */
/**
 * @static
 * @member ConfigurationDebug
 * @type {ConfigurationDebug}
 *
 * Use an empty function to disable the logger.
 *
 * @property {?Function} log If undefined, the default logger will be used.
 * @property {?Function} error If undefined, the default logger will be used.
 */
/**
 * @static
 * @member StaticFiles
 * @type {StaticFiles}
 *
 * @property {string} path The path that will be used to serve the files
 * @property {Array<string>} directory The root directory for the served files
 * @property {?string} subdomain A subdomain is optional here
 * @property {?Array.<string | Function>} middlewares Middlewares
 * @property {?serveStatic.ServeStaticOptions} options Serve Static Options
 * (only works if 'customStaticHandler' is not changed)
 * @property {?rateLimit.Options} rateLimitOptions To disable the rate limiter, set to undefined
 * @property {?boolean} disableIndexRouter If set to false, all the requests (if is not directly to a file)
 * are sent to `index.html` or to `indexFileName` (if set).
 * @property {?Function} customStaticHandler allows you to change the default static file handler
 * e.g.
 * ```js
 * (path: string) => require('connect-gzip-static')(path, { /options here/ })
 * ```
 * @property {?string} indexFileName allows you to change the default index file (default: `index.html`)
 */
/**
 * @static
 * @member StaticFilesSubdomain
 * @type {StaticFilesSubdomain}
 *
 * @property {string} path The path that will be used to serve the files
 * @property {Array<string>} directory The root directory for the served files
 * @property {string} subdomain The subdomain is required.
 * @property {?Array.<string | Function>} middlewares Middlewares
 * @property {?serveStatic.ServeStaticOptions} options Serve Static Options
 * @property {?rateLimit.Options} rateLimitOptions To disable the rate limiter, set to undefined
 */
/**
 * @static
 * @member NotFoundHandler
 * @type {Promise<Response<any>>}
 *
 * @property {Express.Request} req The Express's Request
 * @property {Express.Response} res The Express's Response
 * @property {Express.NextFunction} next The Express's Next Function
 */
/**
 * @static
 * @member ErroHandler
 * @type {Response<any>}
 *
 * @property {Express.Request} req The Express's Request
 * @property {Express.Response} res The Express's Response
 * @property {Express.NextFunction} next The Express's Next Function
 * @property {any} error The error that was caught
 */
//# sourceMappingURL=index.d.ts.map